//******************************************************************************
//  MSP430F22x4 Demo - Transimpedance amplifier with Internal OA
//
//                       MSP430F22x4
//                  ----------------------
// 			   /|\|                   XIN|-
//				|					| 
//				|RST 				|
//				|					|
//				|	   P2.0/OA0I0|<-- '+' OA0
//				|        P2.2/OA0I1|<-- '-' OA0
// 				|        P2.1/OA0O|--> Output OA0
//				|					|
//
//******************************************************************************

#include <msp430.h>


void	initCLKS(void);
void	initTimerA(void);
void	initOA(void);
void	initADC10(void);
void	initGPIO(void);

int main(void){
	
	WDTCTL = WDTPW | WDTHOLD;                 				// Stop WDT
	
	/* Initialize peripherals */
	initCLKS();
	initTimerA();
	initOA();
	initADC10();
	initGPIO();
	
	
	unsigned static const int threshold = 0x1FF;						// 'Bright' reading conversion threshold
	unsigned int ADC_Conversion_Result;									// ADC conversion result
	unsigned int dark_count = 0;												// Counter for number of consecutive dark readings

	while(1){
		ADC10CTL0 |= ENC + ADC10SC;									// ADC sampling and conversion start
		__bis_SR_register(CPUOFF + GIE);        						// Enter LPM0, ADC10_ISR will force exit

		ADC_Conversion_Result = ADC10MEM;							// Get conversion result
		if(ADC_Conversion_Result < threshold) dark_count++;		// If its dark, add to the counter, otherwise reset it
		else dark_count = 0;

		if(dark_count >= 3) P1OUT |= BIT0;								// If there has been 3 or more seconds of darkness turn on LED
		else P1OUT &= ~BIT0;													// Otherwise turn it off


		TAR = 0;										// Reset TAR
		TACTL |= MC_1;							// Start Timer_A Up-Mode, will count to 3k with 3kHz clock
		__bis_SR_register(LPM3_bits);		// Enter LPM3, TIMERA0_ISR will force exit

		/* 1 Second wait until TIMERA0_ISR */

	}
}

/* Initialize system clock */
void	initCLKS(void){
	/* 
     * Basic Clock System Control 2
     * 
     * SELM_0 -- DCOCLK
     * DIVM_0 -- Divide by 1
     * ~SELS -- DCOCLK
     * DIVS_0 -- Divide by 1
     * ~DCOR -- DCO uses internal resistor
     * 
     * Note: ~<BIT> indicates that <BIT> has value zero
     */
    BCSCTL2 = SELM_0 | DIVM_0 | DIVS_0;

    if (CALBC1_1MHZ != 0xFF) {
        /* Follow recommended flow. First, clear all DCOx and MODx bits. Then
         * apply new RSELx values. Finally, apply new DCOx and MODx bit values.
         */
        DCOCTL = 0x00;
        BCSCTL1 = CALBC1_1MHZ;      /* Set DCO to 1MHz */
        DCOCTL = CALDCO_1MHZ;
    }

    /* 
     * Basic Clock System Control 1
     * 
     * XT2OFF -- Disable XT2CLK
     * ~XTS -- Low Frequency
     * DIVA_1 -- Divide by 2
     * 
     * Note: ~XTS indicates that XTS has value zero
     */
    BCSCTL1 |= XT2OFF | DIVA_1;

    /* 
     * Basic Clock System Control 3
     * 
     * XT2S_0 -- 0.4 - 1 MHz
     * LFXT1S_2 -- If XTS = 0, XT1 = VLOCLK ; If XTS = 1, XT1 = 3 - 16-MHz crystal or resonator
     * XCAP_1 -- ~6 pF
     */
    BCSCTL3 = XT2S_0 | LFXT1S_2 | XCAP_1;
}

void	initTimerA(void){

	/* 
     * TACCTL0, Capture/Compare Control Register 0
     * 
     * CM_0 -- No Capture
     * CCIS_0 -- CCIxA
     * ~SCS -- Asynchronous Capture
     * ~SCCI -- Latched capture signal (read)
     * ~CAP -- Compare mode
     * OUTMOD_0 -- PWM output mode: 0 - OUT bit value
     * 
     * Note: ~<BIT> indicates that <BIT> has value zero
     */
    TACCTL0 = CM_0 | CCIS_0 | OUTMOD_0 | CCIE;

    /* TACCR0, Timer_A Capture/Compare Register 0 */
    TACCR0 = 3000;

    /* 
     * TACTL, Timer_A3 Control Register
     * 
     * TASSEL_1 -- ACLK
     * ID_1 -- Divider - /2
     * MC_0 -- Stop Mode
     */
    TACTL = TASSEL_1 | ID_1 | MC_0;
}

/* Initialize operational amplifier */
void	initOA(void){
	/* 
     * OA0 control register 0
     * 
     * OAN_1 -- OAxI1
     * OAP_0 -- OAxI0
     * OAPM_3 -- Fast
     * ~OAADC0 -- OAx output to ADC12 input channel select 0
     * OAADC1 -- OAx output to ADC12 input channel select 1
     * 
     * Note: ~OAADC0 indicates that OAADC0 has value zero
     */
    OA0CTL0 = OAN_1 | OAP_0 | OAPM_3 | OAADC1;
}

/* Initialize ADC10  */
void	initADC10(void){
	/* disable ADC10 during initialization */
    ADC10CTL0 &= ~ENC;

    /* 
     * Control Register 0
     * 
     * ~ADC10SC -- No conversion
     * ~ENC -- Disable ADC
     * ~ADC10IFG -- Clear ADC interrupt flag
     * ADC10IE -- Enable ADC interrupt
     * ADC10ON -- Switch On ADC10
     * ~REFON -- Disable ADC reference generator
     * ~REF2_5V -- Set reference voltage generator = 1.5V
     * ~MSC -- Disable multiple sample and conversion
     * ~REFBURST -- Reference buffer on continuously
     * ~REFOUT -- Reference output off
     * ~ADC10SR -- Reference buffer supports up to ~200 ksps
     * ADC10SHT_2 -- 16 x ADC10CLKs
     * SREF_0 -- VR+ = VCC and VR- = VSS
     * 
     * Note: ~<BIT> indicates that <BIT> has value zero
     */
    ADC10CTL0 = ADC10IE | ADC10ON | ADC10SHT_2 | SREF_0;

    /* 
     * Control Register 1
     * 
     * ~ADC10BUSY -- No operation is active
     * CONSEQ_0 -- Single channel single conversion
     * ADC10SSEL_0 -- ADC10OSC
     * ADC10DIV_0 -- Divide by 1
     * ~ISSH -- Input signal not inverted
     * ~ADC10DF -- ADC10 Data Format as binary
     * SHS_0 -- ADC10SC
     * INCH_1 -- ADC Channel 1
     * 
     * Note: ~<BIT> indicates that <BIT> has value zero
     */
    ADC10CTL1 = CONSEQ_0 | ADC10SSEL_0 | ADC10DIV_0 | SHS_0 | INCH_1;

    /* Analog (Input) Enable Control Register 0 */
    ADC10AE0 = 0x7;


    /* enable ADC10 */
    ADC10CTL0 |= ENC;
}

/*	Initialize GPIO */
void	initGPIO(void){
    /* Port 1 Output Register */
    P1OUT = 0;

    /* Port 1 Direction Register */
    P1DIR = BIT0;
	
	/* Port 2 Output Register */
    P2OUT = 0;
	
    /* Port 2 Direction Register */
    P2DIR = 0;	
	
    /* Port 3 Output Register */
    P3OUT = 0;

    /* Port 3 Direction Register */
    P3DIR = 0;

    /* Port 4 Output Register */
    P4OUT = 0;

    /* Port 4 Direction Register */
    P4DIR = 0;
}

/*
 *  ======== ADC10 Interrupt Service Routine ========
 */
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR_HOOK(void)
{
	__bic_SR_register_on_exit(LPM0_bits);       	// Clear CPUOFF bit from 0(SR), exits LMP0
    
}

/*
 *  ======== Timer_A3 Interrupt Service Routine ======== 
 */
#pragma vector=TIMERA0_VECTOR
__interrupt void TIMERA0_ISR_HOOK(void)
{
	TACTL &= ~MC_0;										// Stop Timer A
	__bic_SR_register_on_exit(LPM3_bits);			// Exit LPM_3
   
}
